/*****************************************************************************
 *
 *            Copyright Andes Technology Corporation 2014
 *                         All Rights Reserved.
 *
 ****************************************************************************/

#ifndef __AG101P_H__
#define __AG101P_H__

#ifndef __ASSEMBLER__
#include <inttypes.h>
#include <nds32_intrinsic.h>
#endif

/*****************************************************************************
 * System clock
 ****************************************************************************/
#define KHz                     1000
#define MHz                     1000000

#define OSCFREQ                 (20 * MHz)
#ifdef CFG_ADDR_16MB
#define CPUFREQ                 (30 * MHz)
#define HCLKFREQ                (CPUFREQ / 1)
#define PCLKFREQ                (CPUFREQ / 2)
#define UCLKFREQ                (PCLKFREQ)
#else
#define CPUFREQ                 (60 * MHz)
#define HCLKFREQ                (CPUFREQ / 2)
#define PCLKFREQ                (CPUFREQ / 4)
#define UCLKFREQ                (PCLKFREQ)
#endif

/*****************************************************************************
 * IRQ Vector
 ****************************************************************************/
#define IRQ_AHBC_NOSLAVE_VECTOR 0
#define IRQ_RESERVE1_VECTOR     1
#define IRQ_SSP1_VECTOR         2
#define IRQ_I2C_VECTOR          3
#define IRQ_CPU_LM_DMA_VECTOR   4       // local memory dma interrupt for EVIC
#define IRQ_SDC_VECTOR          5
#define IRQ_SSP2_VECTOR         6
#define IRQ_UART2_VECTOR        7
#define IRQ_PMU_VECTOR          8
#define IRQ_SWI_VECTOR          9
#define IRQ_CPU_PFM_VECTOR      10
#define IRQ_UART1_VECTOR        11
#define IRQ_RESERVE12_VECTOR    12
#define IRQ_GPIO_VECTOR         13
#define IRQ_TIMER2_VECTOR       14
#define IRQ_TIMER3_VECTOR       15
#define IRQ_WDT_VECTOR          16
#define IRQ_RTCALARM_VECTOR     17
#define IRQ_RTCSECOND_VECTOR    18
#define IRQ_TIMER1_VECTOR       19
#define IRQ_LCDC_VECTOR         20
#define IRQ_DMA_VECTOR          21
#define IRQ_RESERVE22_VECTOR    22
#define IRQ_RESERVE23_VECTOR    23
#define IRQ_APBBRIDGE_VECTOR    24
#define IRQ_ENETMAC_VECTOR      25
#define IRQ_RESERVE26_VECTOR    26
#define IRQ_EXTINT27_VECTOR     27
#define IRQ_EXTINT28_VECTOR     28
#define IRQ_RESERVE29_VECTOR    29
#define IRQ_RESERVE30_VECTOR    30
#define IRQ_RESERVE31_VECTOR    31

#ifndef __ASSEMBLER__

/*****************************************************************************
 * Device Specific Peripheral Registers structures
 ****************************************************************************/

#define __I                     volatile const  /* 'read only' permissions      */
#define __O                     volatile        /* 'write only' permissions     */
#define __IO                    volatile        /* 'read / write' permissions   */

/*****************************************************************************
 * UARTx - AG101P
 ****************************************************************************/
typedef struct {
	union {
		__I  unsigned int RBR;          /* 0x00 Receiver Buffer Register */
		__O  unsigned int THR;          /* 0x00 Transmitter Holding Register */
		__IO unsigned int DLL;          /* 0x00 DLAB = 1, Baud Rate Divisor Latch Least Significant Byte */
	};
	union {
		__IO unsigned int IER;          /* 0x4 Interrupt Enable Register */
		__IO unsigned int DLM;          /* 0x4 DLAB = 1, Baud Rate Divisor Latch Most Significant Byte */
	};
	union {
		__I  unsigned int IIR;          /* 0x8 Interrupt Identification Register */
		__O  unsigned int FCR;          /* 0x8 FIFO Control Register */
		__IO unsigned int PSR;          /* 0x8 DLAB = 1, Prescaler Register */
	};
		__IO unsigned int LCR;          /* 0xC Line Control Register */
		__IO unsigned int MCR;          /* 0x10 Modem Control Register */
	union {
		__I  unsigned int LSR;          /* 0x14 Line Status Register */
		__O  unsigned int TST;          /* 0x14 Testing Register */
	};
	__I  unsigned int MSR;                  /* 0x18 Modem Status Register */
	__IO unsigned int SPR;                  /* 0x1C Scratch Pad Register */
} AG101P_UART_RegDef;

/*****************************************************************************
 * TMR - AG101P
 ****************************************************************************/
typedef struct {
	__IO unsigned int COUNTER;              // TMR Channel Counter Register
	__IO unsigned int RELOAD;               // TMR Channel Reload Register
	__IO unsigned int MATCH1;               // TMR Channel Match1 Register
	__IO unsigned int MATCH2;               // TMR Channel Match2 Register
} TMR_CHANNEL_REG;

typedef struct {
	__IO unsigned int SCALER;               // TMR timer prescaler Register
} TMR_CHANNEL_PRESCALER;

typedef struct {
	TMR_CHANNEL_REG   CHANNEL[3];           /* 0x00 ~ 0x2C Channel #n Registers */
	__IO unsigned int CTRL;                 /* 0x30 TMR Channel Control Register */
	__IO unsigned int INTST;                /* 0x34 Interrupt Status Register */
	__IO unsigned int INTMSK;               /* 0x38 Interrupt Enable Register */
	__I  unsigned int IDREV;                /* 0x3C ID and Revision Register */
	TMR_CHANNEL_PRESCALER PRESCALER[3];     /* 0x40 ~ 0x48 prescaler */
} AG101P_TMR_RegDef;

/*****************************************************************************
 * EXT INTC - AG101P
 ****************************************************************************/
typedef struct {
	__I  unsigned int IRQ_SRC;              /* 0x00 HW1 (IRQ) source register */
	__IO unsigned int IRQ_EN;               /* 0x04 HW1 (IRQ) enable register */
	__IO unsigned int IRQ_INT_CLR;          /* 0x08 HW1 (IRQ) interrupt clear register */
	__IO unsigned int IRQ_TRG_MODE;         /* 0x0C HW1 (IRQ) trigger-mode register */
	__IO unsigned int IRQ_TRG_LVL;          /* 0x10 HW1 (IRQ) trigger-level register */
	__I  unsigned int IRQ_STATUS;           /* 0x14 HW1 (IRQ) status register */
	__IO unsigned int RSV0[2];              /* 0x18 ~ 0x1C Reserved */
	__I  unsigned int FIQ_SRC;              /* 0x20 HW0 (FIQ) source register */
	__IO unsigned int FIQ_EN;               /* 0x24 HW0 (FIQ) enable register */
	__IO unsigned int FIQ_INT_CLR;          /* 0x28 HW0 (FIQ) interrupt clear register */
	__IO unsigned int FIQ_TRG_MODE;         /* 0x2C HW0 (FIQ) trigger-mode register */
	__IO unsigned int FIQ_TRG_LVL;          /* 0x30 HW0 (FIQ) trigger-level register */
	__I  unsigned int FIQ_STATUS;           /* 0x34 HW0 (FIQ) status register */
	__IO unsigned int RSV1[2];              /* 0x38 ~ 0x3C Reserved */
	__IO unsigned int RSV2[4];              /* 0x40 ~ 0x4C Reserved */
	__I  unsigned int REV;                  /* 0x50 Revision register */
	__IO unsigned int RSV3[3];              /* 0x54 ~ 0x5C Reserved */
	__IO unsigned int EVIC_INT_PRI0_7;      /* 0x60 EVIC interrupt priority level configuration for bit0~bit7 */
	__IO unsigned int EVIC_INT_PRI8_15;     /* 0x64 EVIC interrupt priority level configuration for bit8~bit15 */
	__IO unsigned int EVIC_INT_PRI16_23;    /* 0x68 EVIC interrupt priority level configuration for bit16~bit23 */
	__IO unsigned int EVIC_INT_PRI24_31;    /* 0x6C EVIC interrupt priority level configuration for bit24~bit31 */
} AG101P_INTC_RegDef;

/*****************************************************************************
 * Memory Map
 ****************************************************************************/
#if (defined(XIP_MODE) && (defined(CFG_CACHE_ENABLE)))
/*
 * The NTC3 is set to noncache region and NTM3 is mapped to partition 2 (I/O region).
 * Map the I/O address to NTC3 to be uncachable.
 */
#define UNCACHE_MAP(addr)       ((addr) | 0xC0000000)
#else
#define UNCACHE_MAP(addr)       (addr)
#endif

#define _IO_(addr)              UNCACHE_MAP(addr)

#ifdef CFG_ADDR_16MB
#define BTUARTC_BASE            _IO_(0x00F03000)
#define TMRC_BASE               _IO_(0x00F04000)
#define INTC_BASE               _IO_(0x00F08000)
#define STUARTC_BASE            _IO_(0x00F16000)
#else
#define BTUARTC_BASE            _IO_(0x98300000)
#define TMRC_BASE               _IO_(0x98400000)
#define INTC_BASE               _IO_(0x98800000)
#define STUARTC_BASE            _IO_(0x99600000)
#endif

/*****************************************************************************
 * Peripheral declaration
 ****************************************************************************/
#define AG101P_TMR              ((AG101P_TMR_RegDef *) TMRC_BASE)
#define AG101P_INTC             ((AG101P_INTC_RegDef *) INTC_BASE)
#define AG101P_UART1            ((AG101P_UART_RegDef *) BTUARTC_BASE)
#define AG101P_UART2            ((AG101P_UART_RegDef *) STUARTC_BASE)

#define PLAT_UART               AG101P_UART2
#endif	/* __ASSEMBLER__ */

#endif	/* __AG101P_H__ */
